@@ -43,12 +43,12 @@ describe ISO3166::Country do
     expect(country.alpha3).to eq('USA')
   end
 
-  it 'should return 3166 name' do
-    expect(country.name).to eq('United States of America')
+  it 'should return 3166 iso_short_name' do
+    expect(country.iso_short_name).to eq('United States of America')
   end
 
   it 'should return alternate names' do
-    expect(country.names).to eq(['United States', 'Vereinigte Staaten von Amerika', 'États-Unis', 'Estados Unidos', 'アメリカ合衆国', 'Verenigde Staten'])
+    expect(country.unofficial_names).to eq(['United States', 'USA', 'Murica', 'Vereinigte Staaten von Amerika', 'États-Unis', 'Estados Unidos', 'アメリカ合衆国', 'Verenigde Staten'])
   end
 
   it 'should return translations' do
@@ -65,15 +65,7 @@ describe ISO3166::Country do
   end
 
   it 'should return bounds' do
-    expect(country.bounds['northeast']['lat']).to eq(71.5388001)
-  end
-
-  it 'should return the decimal Latitude' do
-    expect(country.latitude_dec).to eq('39.44325637817383')
-  end
-
-  it 'should return the decimal Longitude' do
-    expect(country.longitude_dec).to eq('-98.95733642578125')
+    expect(country.bounds['northeast']['lat']).to eq(71.3577635769)
   end
 
   it 'should return continent' do
@@ -85,8 +77,18 @@ describe ISO3166::Country do
   end
 
   it 'knows when a country does not require postal codes' do
-    ireland = ISO3166::Country.search('IE')
-    expect(ireland.postal_code).to eq(false)
+    antarctica = ISO3166::Country.search('AQ')
+    expect(antarctica.postal_code).to eq(false)
+  end
+
+  it 'knows about the country postal code format' do
+    expect(country.postal_code_format).to_not be_nil
+
+    regex = Regexp.new(country.postal_code_format)
+    expect(regex).to match("12345-6789")
+
+    antarctica = ISO3166::Country.search('AQ')
+    expect(antarctica.postal_code_format).to be_nil
   end
 
   it 'should return region' do
@@ -134,7 +136,7 @@ describe ISO3166::Country do
       before do
         ISO3166::Data.register(
           alpha2: 'BR2',
-          name: 'Brazil',
+          iso_short_name: 'Brazil',
           languages_official: %w(pt-BR),
           translations: {
             'pt-BR' => 'Translation for pt-BR',
@@ -158,7 +160,7 @@ describe ISO3166::Country do
       before do
         ISO3166::Data.register(
           alpha2: 'BR2',
-          name: 'Brazil',
+          iso_short_name: 'Brazil',
           languages_official: %w(pt-BR),
           translations: {
             'pt' => 'Translation for pt'
@@ -223,11 +225,11 @@ describe ISO3166::Country do
     end
 
     it 'should return a hash with all sub divisions' do
-      expect(country.subdivisions.size).to eq(60)
+      expect(country.subdivisions.size).to eq(57)
     end
 
     it 'should be available through states' do
-      expect(country.states.size).to eq(60)
+      expect(country.states.size).to eq(57)
     end
 
     it 'should be a subdivision object' do
@@ -238,6 +240,10 @@ describe ISO3166::Country do
       expect(virginia.name).to eq('Virginia')
     end
 
+    it 'should have a code' do
+      expect(virginia.code).to eq('VA')
+    end
+
     it 'should behave like a hash' do
       expect(virginia['name']).to eq('Virginia')
     end
@@ -309,7 +315,7 @@ describe ISO3166::Country do
     end
 
     it 'should allow to customize each country representation passing a block to the method' do
-      countries = ISO3166::Country.all { |country, data| [data['name'], country, data['country_code']] }
+      countries = ISO3166::Country.all { |country, data| [data['iso_short_name'], country, data['country_code']] }
       expect(countries).to be_an(Array)
       expect(countries.first).to be_an(Array)
       expect(countries.first.size).to eq(3)
@@ -322,7 +328,7 @@ describe ISO3166::Country do
       countries = ISO3166::Country.all_translated('fr')
       expect(countries).to be_an(Array)
       expect(countries.first).to be_a(String)
-      expect(countries.first).to eq('Afghanistan')
+      expect(countries.first).to eq('Aruba')
       # countries missing the desired locale will not be added to the list
       # so all 250 countries may not be returned, 'fr' returns 249, for example
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
@@ -332,9 +338,29 @@ describe ISO3166::Country do
       countries = ISO3166::Country.all_translated
       expect(countries).to be_an(Array)
       expect(countries.first).to be_a(String)
-      expect(countries.first).to eq('Afghanistan')
+      expect(countries.first).to eq('Aruba')
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
     end
+
+    context 'with custom countries' do
+      before do
+        ISO3166::Data.register(
+          alpha2: 'XX',
+          iso_short_name: 'Custom Country',
+          translations: { 'en' => 'Custom Country' }
+        )
+      end
+
+      it 'should include custom registered countries' do
+        custom_country = ISO3166::Country.find_by_alpha2('XX')[1]
+        countries = ISO3166::Country.all_translated
+        expect(countries).to include(custom_country['iso_short_name'])
+      end
+
+      after do
+        ISO3166::Data.unregister('XX')
+      end
+    end
   end
 
   describe 'all_names_with_codes' do
@@ -397,14 +423,31 @@ describe ISO3166::Country do
       countries = ISO3166::Country.new(:de).translation('xxx')
       expect(countries).to be_nil
     end
+
+    context 'should return variant locales' do
+      it 'should return different value for Chinese variants' do
+        ISO3166.configuration.locales = [:'zh-cn', :'zh-hk', :'zh-tw']
+        name_cn = ISO3166::Country['TW'].translation('zh-cn')
+        name_hk = ISO3166::Country['TW'].translation('zh-hk')
+        name_tw = ISO3166::Country['TW'].translation('zh-tw')
+        expect([name_cn, name_hk, name_tw].uniq.size).to eql 3
+      end
+
+      it 'should return different value for Portuguese variants' do
+        ISO3166.configuration.locales = [:pt, :'pt-br']
+        name_pt = ISO3166::Country['SG'].translation('pt')
+        name_br = ISO3166::Country['SG'].translation('pt-br')
+        expect([name_pt, name_br].uniq.size).to eql 2
+      end
+    end
   end
 
   describe 'translations' do
     it 'should return an hash of all country names translated to the selected locale' do
       countries = ISO3166::Country.translations('fr')
       expect(countries).to be_an(Hash)
-      expect(countries.first[0]).to eq('AF')
-      expect(countries.first).to eq(%w(AF Afghanistan))
+      expect(countries.first[0]).to eq('AW')
+      expect(countries.first).to eq(%w(AW Aruba))
       # countries missing the desired locale will not be added to the list
       # so all 250 countries may not be returned, 'fr' returns 249, for example
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
@@ -413,8 +456,8 @@ describe ISO3166::Country do
     it 'should return an hash of all country names in English if no locale is passed' do
       countries = ISO3166::Country.translations
       expect(countries).to be_an(Hash)
-      expect(countries.first[0]).to eq('AF')
-      expect(countries.first).to eq(%w(AF Afghanistan))
+      expect(countries.first[0]).to eq('AW')
+      expect(countries.first).to eq(%w(AW Aruba))
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
     end
   end
@@ -498,29 +541,42 @@ describe ISO3166::Country do
     end
 
     it 'also performs searches with regexps and forces it to ignore case' do
-      spain_data = ISO3166::Country.find_all_by(:names, /Españ/)
+      spain_data = ISO3166::Country.find_all_by(:unofficial_names, /Españ/)
       expect(spain_data).to be_a Hash
       expect(spain_data.keys).to eq(['ES'])
     end
+
+    it 'performs reasonably' do
+      start = Time.now
+      250.times do
+        lookup = ['ZM', 'ZMB', 'Zambia', 'US', 'USA', 'United States'].sample
+        case lookup.length
+        when 2 then ISO3166::Country.find_all_by(:alpha2, lookup)
+        when 3 then ISO3166::Country.find_all_by(:alpha3, lookup)
+        else ISO3166::Country.find_all_by(:iso_short_name, lookup)
+        end
+      end
+      expect(Time.now - start).to be < 1
+    end
   end
 
   describe 'hash finder methods' do
-    context "when search name in 'name'" do
-      subject { ISO3166::Country.find_by_name('Poland') }
+    context "when search name in 'iso_short_name'" do
+      subject { ISO3166::Country.find_by_iso_short_name('Poland') }
       it 'should return' do
         expect(subject.first).to eq('PL')
       end
     end
 
-    context "when search lowercase name in 'name'" do
-      subject { ISO3166::Country.find_by_name('poland') }
+    context "when search lowercase name in 'iso_short_name'" do
+      subject { ISO3166::Country.find_by_iso_short_name('poland') }
       it 'should return' do
         expect(subject.first).to eq('PL')
       end
     end
 
-    context "when search name with comma in 'name'" do
-      subject { ISO3166::Country.find_by_name(country_name) }
+    context "when search name with comma in 'iso_short_name'" do
+      subject { ISO3166::Country.find_by_iso_short_name(country_name) }
 
       context 'with Republic of Korea' do
         let(:country_name) { 'Korea, Republic of' }
@@ -545,7 +601,7 @@ describe ISO3166::Country do
     end
 
     context 'when search lowercase multibyte name found' do
-      subject { ISO3166::Country.find_country_by_name('россия') }
+      subject { ISO3166::Country.find_country_by_unofficial_names('россия') }
 
       it 'should be a country instance' do
         expect(subject).to be_a(ISO3166::Country)
@@ -554,7 +610,7 @@ describe ISO3166::Country do
     end
 
     context 'when search lowercase multibyte name found' do
-      subject { ISO3166::Country.find_country_by_name(/россия/) }
+      subject { ISO3166::Country.find_country_by_unofficial_names(/россия/) }
 
       it 'should be a country instance' do
         expect(subject).to be_a(ISO3166::Country)
@@ -563,7 +619,7 @@ describe ISO3166::Country do
     end
 
     context 'when accents are not used' do
-      subject { ISO3166::Country.find_country_by_name('emirats Arabes Unis') }
+      subject { ISO3166::Country.find_country_by_unofficial_names('emirats Arabes Unis') }
 
       it 'should be a country instance' do
         expect(subject).to be_a(ISO3166::Country)
@@ -571,8 +627,20 @@ describe ISO3166::Country do
       end
     end
 
-    context "when search name in 'names'" do
-      subject { ISO3166::Country.find_by_name('Polonia') }
+    context "when search name in 'unofficial_names'" do
+      subject { ISO3166::Country.find_by_unofficial_names('Polonia') }
+      it 'should return' do
+        expect(subject.first).to eq('PL')
+      end
+    end
+
+    context "when search name in 'translated_names'" do
+      before do
+        ISO3166.configure do |config|
+          config.locales = [:bs]
+        end
+      end
+      subject { ISO3166::Country.find_by_translated_names('Poljska') }
       it 'should return' do
         expect(subject.first).to eq('PL')
       end
@@ -605,7 +673,7 @@ describe ISO3166::Country do
 
   describe 'country finder methods' do
     context 'when search name found' do
-      let(:uk) { ISO3166::Country.find_country_by_name('United Kingdom') }
+      let(:uk) { ISO3166::Country.find_country_by_unofficial_names('United Kingdom') }
 
       it 'should be a country instance' do
         expect(uk).to be_a(ISO3166::Country)
@@ -614,7 +682,7 @@ describe ISO3166::Country do
     end
 
     context 'when search lowercase name found' do
-      let(:uk) { ISO3166::Country.find_country_by_name('united kingdom') }
+      let(:uk) { ISO3166::Country.find_country_by_unofficial_names('united kingdom') }
 
       it 'should be a country instance' do
         expect(uk).to be_a(ISO3166::Country)
@@ -623,7 +691,7 @@ describe ISO3166::Country do
     end
 
     context 'when the search term contains comma' do
-      let(:korea) { ISO3166::Country.find_country_by_name('Korea, Republic of') }
+      let(:korea) { ISO3166::Country.find_country_by_unofficial_names('Korea, Republic of') }
 
       it 'should be a country instance' do
         expect(korea).to be_a(ISO3166::Country)
@@ -662,7 +730,7 @@ describe ISO3166::Country do
       end
     end
     context 'when search name not found' do
-      let(:bogus) { ISO3166::Country.find_country_by_name('Does not exist') }
+      let(:bogus) { ISO3166::Country.find_country_by_unofficial_names('Does not exist') }
 
       it 'should be a country instance' do
         expect(bogus).to eq(nil)
@@ -671,7 +739,7 @@ describe ISO3166::Country do
 
     # Spot checks #243
     context 'when search name not found' do
-      let(:belgium) { ISO3166::Country.find_country_by_name('Belgium') }
+      let(:belgium) { ISO3166::Country.find_country_by_unofficial_names('Belgium') }
 
       it 'should be a country instance' do
         expect(belgium.alpha2).to eq('BE')
@@ -680,7 +748,7 @@ describe ISO3166::Country do
 
     # Spot checks #240
     context 'when search name not found' do
-      let(:canada) { ISO3166::Country.find_country_by_name('Canada') }
+      let(:canada) { ISO3166::Country.find_country_by_unofficial_names('Canada') }
 
       it 'should be a country instance' do
         expect(canada.alpha2).to eq('CA')
@@ -689,7 +757,7 @@ describe ISO3166::Country do
 
     # Spot checks #241
     context 'when search name not found' do
-      let(:israel) { ISO3166::Country.find_country_by_name('Israel') }
+      let(:israel) { ISO3166::Country.find_country_by_unofficial_names('Israel') }
 
       it 'should be a country instance' do
         expect(israel.alpha2).to eq('IL')
@@ -698,7 +766,7 @@ describe ISO3166::Country do
 
     # Spot checks #241
     context 'when search name not found' do
-      let(:israel) { ISO3166::Country.find_by_name('Israel') }
+      let(:israel) { ISO3166::Country.find_by_iso_short_name('Israel') }
 
       it 'should be a country instance' do
         expect(israel[0]).to eq('IL')
@@ -707,7 +775,7 @@ describe ISO3166::Country do
 
     # Spot checks #241
     context 'when search name not found' do
-      let(:israel) { ISO3166::Country.find_all_by(:name, 'Israel') }
+      let(:israel) { ISO3166::Country.find_all_by(:iso_short_name, 'Israel') }
 
       it 'should be a country instance' do
         expect(israel.size).to eq(1)
@@ -740,10 +808,49 @@ describe ISO3166::Country do
     end
   end
 
+  describe 'finder methods respond_to_missing?' do
+    subject { ISO3166::Country.respond_to?(method_name) }
+    describe 'find_all_by' do
+      context 'find by a valid Country attribute' do
+        let(:method_name) { :find_all_by_currency }
+        it { is_expected.to be true }
+      end
+
+      context 'find by an invalid attribute' do
+        let(:method_name) { :find_all_by_invalid }
+        it { is_expected.to be false }
+      end
+    end
+
+    describe 'hash finder methods' do
+      context 'find by a valid Country attribute' do
+        let(:method_name) { :find_by_iso_short_name }
+        it { is_expected.to be true }
+      end
+
+      context 'find by an invalid attribute' do
+        let(:method_name) { :find_by_invalid }
+        it { is_expected.to be false }
+      end
+    end
+
+    describe 'country finder methods' do
+      context 'find country by a valid Country attribute' do
+        let(:method_name) { :find_country_by_alpha3 }
+        it { is_expected.to be true }
+      end
+
+      context 'find by an invalid attribute' do
+        let(:method_name) { :find_country_by_invalid }
+        it { is_expected.to be false }
+      end
+    end
+  end
+
   describe 'names in Data' do
     it 'should be unique (to allow .find_by_name work properly)' do
       names = ISO3166::Data.cache.map do |_k, v|
-        [v['name'], v['unofficial_names']].flatten.uniq
+        [v['iso_short_name'], v['unofficial_names']].flatten.uniq
       end.flatten
 
       expect(names.size).to eq(names.uniq.size)
@@ -798,6 +905,23 @@ describe ISO3166::Country do
     end
   end
 
+  describe 'in_esm?' do
+    let(:netherlands) { ISO3166::Country.search('NL') }
+    let(:switzerland) { ISO3166::Country.search('CH') }
+
+    it 'should return false for countries without esm_member or eea_member flag' do
+      expect(country.in_esm?).to be_falsey
+    end
+
+    it 'should return true for countries with eea_member flag set to true' do
+      expect(netherlands.in_esm?).to be_truthy
+    end
+
+    it 'should return true for countries with esm_member flag set to true' do
+      expect(switzerland.in_esm?).to be_truthy
+    end
+  end
+
   describe 'gec' do
     it 'should return the country\'s GEC code' do
       expect(ISO3166::Country.new('NA').gec).to eql 'WA'
@@ -809,7 +933,7 @@ describe ISO3166::Country do
   end
 
   describe 'to_s' do
-    it 'should return the country name' do
+    it 'should return the country iso_short_name' do
       expect(ISO3166::Country.new('GB').to_s).to eq('United Kingdom of Great Britain and Northern Ireland')
     end
   end
@@ -850,30 +974,30 @@ describe ISO3166::Country do
 
   describe 'Added country names to search by' do
     it 'should return country code for Democratic Republic of the Congo' do
-      expect(ISO3166::Country.find_country_by_name('Democratic Republic of the Congo').alpha2).to eq 'CD'
+      expect(ISO3166::Country.find_country_by_unofficial_names('Democratic Republic of the Congo').alpha2).to eq 'CD'
     end
     it 'should return country code for Ivory Coast' do
-      expect(ISO3166::Country.find_country_by_name('Ivory Coast').alpha2).to eq 'CI'
+      expect(ISO3166::Country.find_country_by_unofficial_names('Ivory Coast').alpha2).to eq 'CI'
     end
     it 'should return Pakistan code for Guinea Bissau' do
-      expect(ISO3166::Country.find_country_by_name('Guinea Bissau').alpha2).to eq 'GW'
+      expect(ISO3166::Country.find_country_by_unofficial_names('Guinea Bissau').alpha2).to eq 'GW'
     end
     it 'should return Pakistan code for St Kitts and Nevis' do
-      expect(ISO3166::Country.find_country_by_name('St Kitts and Nevis').alpha2).to eq 'KN'
+      expect(ISO3166::Country.find_country_by_unofficial_names('St Kitts and Nevis').alpha2).to eq 'KN'
     end
     it 'should return Pakistan code for St Lucia' do
-      expect(ISO3166::Country.find_country_by_name('St Lucia').alpha2).to eq 'LC'
-      expect(ISO3166::Country.find_country_by_name('St. Lucia').alpha2).to eq 'LC'
+      expect(ISO3166::Country.find_country_by_unofficial_names('St Lucia').alpha2).to eq 'LC'
+      expect(ISO3166::Country.find_country_by_unofficial_names('St. Lucia').alpha2).to eq 'LC'
     end
     it 'should return Pakistan code for Turks and Caicos' do
-      expect(ISO3166::Country.find_country_by_name('Turks and Caicos').alpha2).to eq 'TC'
+      expect(ISO3166::Country.find_country_by_unofficial_names('Turks and Caicos').alpha2).to eq 'TC'
     end
     it 'should return Pakistan code for St Vincent Grenadines' do
-      expect(ISO3166::Country.find_country_by_name('St Vincent Grenadines').alpha2).to eq 'VC'
-      expect(ISO3166::Country.find_country_by_name('St. Vincent Grenadines').alpha2).to eq 'VC'
+      expect(ISO3166::Country.find_country_by_unofficial_names('St Vincent Grenadines').alpha2).to eq 'VC'
+      expect(ISO3166::Country.find_country_by_unofficial_names('St. Vincent Grenadines').alpha2).to eq 'VC'
     end
     it 'should return country code for Palestinian Authority' do
-      expect(ISO3166::Country.find_country_by_name('Palestinian Authority').alpha2).to eq 'PS'
+      expect(ISO3166::Country.find_country_by_unofficial_names('Palestinian Authority').alpha2).to eq 'PS'
     end
   end
 
@@ -891,4 +1015,24 @@ describe ISO3166::Country do
       expect(countries.all? { |country| country.un_locode.length == 2 }).to be
     end
   end
+
+  describe '.pluck' do
+    let(:args) { [] }
+
+    subject { ISO3166::Country.pluck(*args) }
+
+    it "returns empty arrays" do
+      expect(subject.first).to be_empty
+      expect(subject.last).to be_empty
+    end
+
+    context "when asking for alpha2, alpha3 and iso_short_name" do
+      let(:args) { [:alpha2, :alpha3, :iso_short_name] }
+
+      it "returns the correct values" do
+        expect(subject.first).to eq(["AD", "AND", "Andorra"])
+        expect(subject.last).to eq(["ZW", "ZWE", "Zimbabwe"])
+      end
+    end
+  end
 end
